#ifndef _DS_TIME_H
#define _DS_TIME_H

#include <dslib.h>
#include <dsmalloc.h>

#ifdef TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else 
# ifdef HAVE_SYS_TIME_H
# include <sys/time.h>
# error "time.h required"
# else
# include <time.h>
# endif
#endif

#ifdef _WIN32
# include <windows.h>
#endif /* _WIN32 */

typedef struct DSDurationT *DSDuration;
typedef struct DSTimeT *DSTime;
typedef const struct DSDurationT *DSDurationConst;
typedef const struct DSTimeT *DSTimeConst;

struct DSDurationT {
	// This part of structure must correspond to system "struct timeval"
    time_t s;
    suseconds_t us;

	// Information about Years, Months, Days should be here
    
	// Additional Information
    DSMallocContext malloc;
};
typedef struct DSDurationT DSDurationS;

struct DSTimeT {
	// This part of structure must correspond to system "struct timeval"
    time_t s;
    suseconds_t us;
    
	// Additional information
    long offset;
    DSMallocContext malloc;
};
typedef struct DSTimeT DSTimeS;

int dsLibTimeInit();
void dsLibTimeFree();


DSTime dsTimeCreate(DSMallocContext mctx);
void dsTimeDestroy(DSTime time);
DSDuration dsDurationCreate(DSMallocContext mctx);
void dsDurationDestroy(DSDuration duration);

int dsTimeCurrentTime(DSTime time);
int dsTimeCurrentOffset(DSTime time);
int dsTimeSetOffset(DSTime time, long offset);

int dsTimeZeroTime(DSTime time);
int dsTimeIsZero(DSTime time);
int dsDurationZeroDuration(DSDuration duration);
int dsDurationIsZero(DSDuration duration);

#ifdef HAVE_SYS_TIME_H
int dsTimeImportUnixTimeval(DSTime time, struct timeval *newtime);
#endif /* HAVE_SYS_TIME_H */
int dsDurationImportSeconds(DSDuration duration, time_t s);
int dsDurationImportMilliseconds(DSDuration duration, unsigned long ms);
int dsDurationImportMicroseconds(DSDuration duration, unsigned long us);

#ifdef HAVE_TIMESPEC
int dsDurationImportTimespec(DSDuration duration, struct timespec *ts);
int dsDurationExportTimespec(DSDuration duration, struct timespec *ts);
int dsTimeImportTimespec(DSTime time, struct timespec *ts);
int dsTimeExportTimespec(DSTime time, struct timespec *ts);
#endif /* HAVE_TIMESPEC */

int dsTimeExportLocaltime(DSTime time, struct tm *res);
int dsTimeExportGmtime(DSTime time, struct tm *res);
time_t dsTimeGetUnixTime(DSTime time);

int dsTimeCalcDuration(DSDuration duration, DSTime stime, DSTime etime);
int dsTimeCalcCurrentDuration(DSDuration duration, DSTime time);
int dsTimeCalcPassedDuration(DSDuration duration, DSTime time);
int dsTimeDiffSeconds(DSTime stime, DSTime etime);
int dsTimeDiffCurrentSeconds(DSTime time);
int dsTimeDiffMilliseconds(DSTime stime, DSTime etime);
int dsTimeDiffCurrentMilliseconds(DSTime time);
int dsTimeDiffMicroseconds(DSTime stime, DSTime etime);
int dsTimeDiffCurrentMicroseconds(DSTime time);

int dsTimeAdd(DSTime time, DSDuration duration);
int dsTimeAddSeconds(DSTime time, time_t sec);
int dsTimeAddMilliseconds(DSTime time, unsigned long ms);
int dsTimeAddMicroseconds(DSTime time, unsigned long us);
int dsTimeSub(DSTime time, DSDuration duration);
int dsTimeSubSeconds(DSTime time, time_t s);
int dsTimeSubMilliseconds(DSTime time, unsigned long ms);
int dsTimeSubMicroseconds(DSTime time, unsigned long us);

int dsDurationAdd(DSDuration duration, DSDuration addition);
int dsDurationAddSeconds(DSDuration duration, time_t s);
int dsDurationAddMilliseconds(DSDuration duration, unsigned long ms);
int dsDurationAddMicroseconds(DSDuration duration, unsigned long us);
int dsDurationSub(DSDuration duration, DSDuration substraction);
int dsDurationSubSeconds(DSDuration duration, time_t s);
int dsDurationSubMilliseconds(DSDuration duration, unsigned long ms);
int dsDurationSubMicroseconds(DSDuration duration, unsigned long us);
int dsDurationMultiply(DSDuration duration, double multiplier);
int dsDurationIntegerMultiply(DSDuration duration, unsigned long multiplier);
int dsDurationIntegerDivide(DSDuration duration, unsigned long divieder);


int dsTimeCopy(DSTime dst, DSTime src);
DSTime dsTimeCreateCopy(DSMallocContext mctx, DSTime tm);
int dsDurationCopy(DSDuration dst, DSDuration src);
DSDuration dsDurationCreateCopy(DSMallocContext mctx, DSDuration dur);

#define dsTimeImport(time, buffer) dsTimeSetDateTime(time, buffer)
#define dsTimeExport(time, buffer) dsTimeGetDateTime(time, buffer)
#define dsTimeSet(time, buffer) dsTimeSetDateTime(time, buffer)
#define dsTimeGet(time, buffer) dsTimeGetDateTime(time, buffer)
int dsTimeSetDateTime(DSTime time, const char *buffer);
char *dsTimeGetDateTime(DSTime time, char *buffer);
char *dsTimeGetDate(DSTime time, char *buffer);
char *dsTimeGetTime(DSTime time, char *buffer);

#define dsDurationImport(duration, buffer) dsDurationSet(duration, buffer)
#define dsDurationExport(duration, buffer) dsDurationGet(duration, buffer)
int dsDurationSet(DSDuration duration, const char *buffer);
char *dsDurationGet(DSDuration duration, char *buffer);
char *dsTimeGetDuration(DSTime stime, DSTime etime, char *buffer);

int dsTimeCmp(DSTime time1, DSTime time2);
int dsTimeCurrentCmp(DSTime time);
int dsDurationCmp(DSDuration duration1, DSDuration duration2);

unsigned long dsDurationGetSeconds(DSDuration duration);
unsigned long dsDurationGetMilliseconds(DSDuration duration);
unsigned long dsDurationGetMicroseconds(DSDuration duration);

int dsDurationCeilMicroseconds(DSDuration duration, unsigned long us);
int dsDurationCeilMilliseconds(DSDuration duration, unsigned long ms);
int dsDurationCeilSeconds(DSDuration duration, unsigned long s);

int dsTimeCeil(DSTime time, DSDuration duration);

#ifdef _WIN32
time_t dsTimeConvert_WIN32_FILETIME_to_UnixTime(FILETIME FileTime);
# ifndef HAVE_GETTIMEOFDAY
int gettimeofday(struct timeval *tv, void *fake);
# endif /* HAVE_GETTIMEOFDAY */
#endif /* _WIN32 */

#include <dssignal.h>
#define dsWait(duration) dsSignalWait(NULL, duration)
#define dsWaitRT(duration) dsSignalWaitRT(NULL, duration)
#define dsWaitUntil(time) dsSignalWaitUntil(NULL, time)
#define dsWaitRTUntil(time) dsSignalWaitRTUntil(NULL, time)

int dsWaitSeconds(time_t s);
int dsWaitMilliseconds(unsigned long ms);
int dsWaitMicroseconds(unsigned long us);
int dsWaitMicrosecondsRT(unsigned long us);

#endif /* _DS_TIME_H */
